import QtQuick
import QtQuick.Controls
import QtQuick.Layouts
import QtWebEngine

import WebPage
import Core
import I18n

import "../core"
import "../widgets"

BaseForm {
    id: root
    objectName: "webPageForm"
    Style.class: "webPageForm"
    property Logger log: Utils.createLogger("WebPage")
    property point keyboardPosition: webPageBox.mapToGlobal(
                                         (webPageBox.width - Style.parameter("geometry.keyboardWindow.width", 1010)) / 2,
                                         (webPageBox.height - Style.parameter("geometry.keyboardWindow.height", 720)))

    signal hideKeyboard()
    signal moveKeyboard(x: var, y: var)

    Connections {
        target: Qt.inputMethod
        function onVisibleChanged() {
            if (Qt.inputMethod.visible) {
                webView.runJavaScript("document.activeElement.tagName === 'INPUT' && document.activeElement.type === 'email'",
                                      function (isEmail) {
                                          root.floatKeyboardLayoutType = isEmail ? "email" : "main"
                                      })
            }
            else {
                hideKeyboard()
            }
        }
    }

    Context.context {
        inputSources: InputSources.Scanner
    }

    focusPolicy: Qt.ClickFocus

    GridLayout {
        objectName: "mainLayout"
        anchors.fill: parent
        columns: 1
        rowSpacing: 30

        SGroupBox {
            id: webPageBox
            objectName: "webPageBox"
            padding: 40

            contentItem: Item {
                ColumnLayout {
                    objectName: "webPageLayout"
                    anchors.fill: parent
                    spacing: 30

                    RowLayout {
                        id: webPageControlPanelLayout
                        objectName: "webPageControlPanelLayout"
                        spacing: 50

                        RowLayout {
                            objectName: "webPageControlPanelButtonsLayout"
                            spacing: 20

                            ActionButton {
                                id: webPageBackButton
                                objectName: "webPageBackButton"
                                Style.class: "webPageControlButton"
                                icon: {
                                    if (!enabled)
                                        return Style.parameter("icon.webPage.back_disabled")
                                    return down ? Style.parameter("icon.webPage.back_pressed") : Style.parameter("icon.webPage.back")
                                }
                                enabled: webView.canGoBack
                                onClicked: webView.goBack()

                                Layout.preferredWidth: height
                                Layout.preferredHeight: webPageControlPanelLayout.height
                            }

                            ActionButton {
                                id: webPageForwardButton
                                objectName: "webPageForwardButton"
                                Style.class: "webPageControlButton"
                                icon: {
                                    if (!enabled)
                                        return Style.parameter("icon.webPage.forward_disabled")
                                    return down ? Style.parameter("icon.webPage.forward_pressed") : Style.parameter("icon.webPage.forward")
                                }
                                enabled: webView.canGoForward
                                onClicked: webView.goForward()

                                Layout.preferredWidth: height
                                Layout.preferredHeight: webPageControlPanelLayout.height
                            }

                            ActionButton {
                                id: webPageReloadButton
                                objectName: "webPageReloadButton"
                                Style.class: "webPageControlButton"
                                icon: {
                                    if (webView.loading)
                                        return down ? Style.parameter("icon.webPage.stop_pressed") : Style.parameter("icon.webPage.stop")
                                    return down ? Style.parameter("icon.webPage.reload_pressed") : Style.parameter("icon.webPage.reload")
                                }
                                onClicked: webView.loading ? webView.stop() : webView.reload()

                                Layout.preferredWidth: height
                                Layout.preferredHeight: webPageControlPanelLayout.height
                            }
                        }

                        SLabel {
                            id: webPageTitleLabel
                            objectName: "webPageTitleLabel"
                            Style.class: "webPageTitleLabel"
                            text: webView.title
                            elide: Text.ElideRight

                            Layout.fillWidth: true
                        }

                        Layout.preferredHeight: 56
                        Layout.fillWidth: true
                    }

                    SProgressBar {
                        id: webPageProgressBar
                        objectName: "webPageProgressBar"
                        Style.class: "webPageProgressBar"
                        value: webView.loadProgress
                        content.visible: webView.loading

                        Layout.preferredHeight: 1
                        Layout.fillWidth: true
                    }

                    WebEngineView {
                        id: webView
                        objectName: "webView"
                        url: WebPageState.url

                        onContextMenuRequested: {
                            request.accepted = true
                        }
                        onLoadProgressChanged: {
                            log?.trace("Изменение прогресса загрузки", [{name: "Прогресс", value: loadProgress}])
                        }
                        onUrlChanged: {
                            log?.info("Изменение адреса", [{name: "Адрес", value: url}])
                        }
                        onLoadingChanged: {
                            if (loadingInfo.status == WebEngineView.LoadFailedStatus) {
                                log?.error("Ошибка загрузки страницы")
                                removeContext()
                            }
                            if (loadingInfo.status == WebEngineView.LoadSucceededStatus)
                                webView.forceActiveFocus()
                        }
                        onNavigationRequested: function(request) {
                            var requestedUrl = request.url
                            var allow = WebPageState.allow(requestedUrl)
                            if (!allow) {
                                log?.debug("Заблокирована загрузка", [{name: "Путь", value: requestedUrl}])
                            }
                            request.action = allow ? WebEngineNavigationRequest.AcceptRequest : WebEngineNavigationRequest.IgnoreRequest
                        }

                        Layout.fillWidth: true
                        Layout.fillHeight: true
                    }
                }
            }

            Layout.fillWidth: true
            Layout.fillHeight: true
        }

        ActionButton {
            id: webPageCloseButton
            objectName: "webPageCloseButton"
            Style.class: "webPageCloseButton"
            icon: Style.parameter("icon.webPage.close")
            Tr { code: "webPageCloseButton" }
            onClicked: removeContext()

            Layout.fillWidth: true
            Layout.preferredHeight: 80
        }
    }

    onActivated: {
        root.moveKeyboard(root.keyboardPosition.x, root.keyboardPosition.y)
    }
}
